﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.JobBase;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using System.Drawing;

namespace HeuristicLab.HiveEngine {
  [StorableClass]
  public class OperationJob : AbstractJob {
    [Storable]
    SequentialEngine.SequentialEngine engine;

    [Storable]
    protected IOperation operation;
    public IOperation Operation {
      get { return operation; }
      set { operation = value; }
    }

    public override TimeSpan ExecutionTime {
      get { return engine.ExecutionTime; }
    }

    public override ExecutionState ExecutionState {
      get { return engine.ExecutionState; }
    }

    #region constructors and cloning
    public OperationJob() { }
    public OperationJob(IOperation operation) {
      this.operation = operation;
      this.engine = new SequentialEngine.SequentialEngine(); 
      RegisterEngineEvents();
    }

    [StorableConstructor]
    protected OperationJob(bool deserializing) : base(deserializing) { }
    protected OperationJob(OperationJob original, Cloner cloner)
      : base(original, cloner) {
      this.engine = cloner.Clone(original.engine);
      this.operation = cloner.Clone(original.operation);
      RegisterEngineEvents();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new OperationJob(this, cloner);
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEngineEvents();
    }
    #endregion

    public override bool IsParallelizable {
      get { return false; }
    }

    public override void Run() {
      throw new NotImplementedException();
    }

    public override void Prepare() { }
    public override void Start() {
      engine.Prepare(operation);
      engine.Start();
    }

    public override void Stop() {
      engine.Stop();
    }

    public override void Resume(IEnumerable<IJob> childJobs) {
      throw new NotImplementedException();
    }

    private void RegisterEngineEvents() {
      engine.Stopped += new EventHandler(engine_Stopped);
      engine.ExceptionOccurred += new EventHandler<EventArgs<Exception>>(engine_ExceptionOccurred);
    }

    void engine_ExceptionOccurred(object sender, EventArgs<Exception> e) {
      OnJobFailed(e);
    }

    void engine_Stopped(object sender, EventArgs e) {
      OnJobStopped();
    }

    public override bool CanChangeDescription {
      get { return false; }
    }

    public override bool CanChangeName {
      get { return false; }
    }

    public override string Description {
      get { return operation is IAtomicOperation ? ((IAtomicOperation)operation).Operator.Name : "OperationCollection"; }
      set { throw new NotSupportedException(); }
    }

    public override string Name {
      get { return operation is IAtomicOperation ? ((IAtomicOperation)operation).Operator.Name : "OperationCollection"; }
      set { throw new NotSupportedException(); }
    }

    public override event EventHandler NameChanged;
    public override event EventHandler<CancelEventArgs<string>> NameChanging;
    public override event EventHandler ToStringChanged;
    public override event EventHandler DescriptionChanged;

    public override string ItemDescription {
      get { return Description; }
    }

    public override Image ItemImage {
      get { return HeuristicLab.Common.Resources.VS2008ImageLibrary.Operator; }
    }

    public override event EventHandler ItemImageChanged;

    public override string ItemName {
      get { return "OperationJob"; }
    }

    public override Version ItemVersion {
      get { return null; }
    }
  }
}
